@@ -12,7 +12,10 @@ from dataclasses import dataclass
 from heapq import heappop, heappush
 
 from sqlglot import Dialect, expressions as exp
-from sqlglot.helper import ensure_list
+from sqlglot.helper import seq_get
+
+if t.TYPE_CHECKING:
+    from sqlglot.dialects.dialect import DialectType
 
 
 @dataclass(frozen=True)
@@ -33,7 +36,8 @@ class Remove:
 class Move:
     """Indicates that an existing node's position within the tree has changed"""
 
-    expression: exp.Expression
+    source: exp.Expression
+    target: exp.Expression
 
 
 @dataclass(frozen=True)
@@ -53,7 +57,8 @@ class Keep:
 
 
 if t.TYPE_CHECKING:
-    T = t.TypeVar("T")
+    from sqlglot._typing import T
+
     Edit = t.Union[Insert, Remove, Move, Update, Keep]
 
 
@@ -61,6 +66,8 @@ def diff(
     source: exp.Expression,
     target: exp.Expression,
     matchings: t.List[t.Tuple[exp.Expression, exp.Expression]] | None = None,
+    delta_only: bool = False,
+    copy: bool = True,
     **kwargs: t.Any,
 ) -> t.List[Edit]:
     """
@@ -87,7 +94,12 @@ def diff(
         matchings: the list of pre-matched node pairs which is used to help the algorithm's
             heuristics produce better results for subtrees that are known by a caller to be matching.
             Note: expression references in this list must refer to the same node objects that are
-            referenced in source / target trees.
+            referenced in the source / target trees.
+        delta_only: excludes all `Keep` nodes from the diff.
+        copy: whether to copy the input expressions.
+            Note: if this is set to false, the caller must ensure that there are no shared references
+            in the two trees, otherwise the diffing algorithm may produce unexpected behavior.
+        kwargs: additional arguments to pass to the ChangeDistiller instance.
 
     Returns:
         the list of Insert, Remove, Move, Update and Keep objects for each node in the source and the
@@ -102,12 +114,12 @@ def diff(
     ) -> t.Dict[int, exp.Expression]:
         return {
             id(old_node): new_node
-            for (old_node, _, _), (new_node, _, _) in zip(original.walk(), copy.walk())
+            for old_node, new_node in zip(original.walk(), copy.walk())
             if id(old_node) in matching_ids
         }
 
-    source_copy = source.copy()
-    target_copy = target.copy()
+    source_copy = source.copy() if copy else source
+    target_copy = target.copy() if copy else target
 
     node_mappings = {
         **compute_node_mappings(source, source_copy),
@@ -115,16 +127,28 @@ def diff(
     }
     matchings_copy = [(node_mappings[id(s)], node_mappings[id(t)]) for s, t in matchings]
 
-    return ChangeDistiller(**kwargs).diff(source_copy, target_copy, matchings=matchings_copy)
+    return ChangeDistiller(**kwargs).diff(
+        source_copy,
+        target_copy,
+        matchings=matchings_copy,
+        delta_only=delta_only,
+    )
 
 
-LEAF_EXPRESSION_TYPES = (
+# The expression types for which Update edits are allowed.
+UPDATABLE_EXPRESSION_TYPES = (
+    exp.Alias,
     exp.Boolean,
+    exp.Column,
     exp.DataType,
-    exp.Identifier,
+    exp.Lambda,
     exp.Literal,
+    exp.Table,
+    exp.Window,
 )
 
+IGNORED_LEAF_EXPRESSION_TYPES = (exp.Identifier,)
+
 
 class ChangeDistiller:
     """
@@ -133,16 +157,17 @@ class ChangeDistiller:
     Chawathe et al. described in http://ilpubs.stanford.edu:8090/115/1/1995-46.pdf.
     """
 
-    def __init__(self, f: float = 0.6, t: float = 0.6) -> None:
+    def __init__(self, f: float = 0.6, t: float = 0.6, dialect: DialectType = None) -> None:
         self.f = f
         self.t = t
-        self._sql_generator = Dialect().generator()
+        self._sql_generator = Dialect.get_or_raise(dialect).generator()
 
     def diff(
         self,
         source: exp.Expression,
         target: exp.Expression,
         matchings: t.List[t.Tuple[exp.Expression, exp.Expression]] | None = None,
+        delta_only: bool = False,
     ) -> t.List[Edit]:
         matchings = matchings or []
         pre_matched_nodes = {id(s): id(t) for s, t in matchings}
@@ -151,46 +176,63 @@ class ChangeDistiller:
 
         self._source = source
         self._target = target
-        self._source_index = {id(n): n for n, *_ in self._source.bfs()}
-        self._target_index = {id(n): n for n, *_ in self._target.bfs()}
+        self._source_index = {
+            id(n): n for n in self._source.bfs() if not isinstance(n, IGNORED_LEAF_EXPRESSION_TYPES)
+        }
+        self._target_index = {
+            id(n): n for n in self._target.bfs() if not isinstance(n, IGNORED_LEAF_EXPRESSION_TYPES)
+        }
         self._unmatched_source_nodes = set(self._source_index) - set(pre_matched_nodes)
         self._unmatched_target_nodes = set(self._target_index) - set(pre_matched_nodes.values())
         self._bigram_histo_cache: t.Dict[int, t.DefaultDict[str, int]] = {}
 
-        matching_set = self._compute_matching_set() | {(s, t) for s, t in pre_matched_nodes.items()}
-        return self._generate_edit_script(matching_set)
+        matching_set = self._compute_matching_set() | set(pre_matched_nodes.items())
+        return self._generate_edit_script(dict(matching_set), delta_only)
 
-    def _generate_edit_script(self, matching_set: t.Set[t.Tuple[int, int]]) -> t.List[Edit]:
+    def _generate_edit_script(self, matchings: t.Dict[int, int], delta_only: bool) -> t.List[Edit]:
         edit_script: t.List[Edit] = []
         for removed_node_id in self._unmatched_source_nodes:
             edit_script.append(Remove(self._source_index[removed_node_id]))
         for inserted_node_id in self._unmatched_target_nodes:
             edit_script.append(Insert(self._target_index[inserted_node_id]))
-        for kept_source_node_id, kept_target_node_id in matching_set:
+        for kept_source_node_id, kept_target_node_id in matchings.items():
             source_node = self._source_index[kept_source_node_id]
             target_node = self._target_index[kept_target_node_id]
-            if not isinstance(source_node, LEAF_EXPRESSION_TYPES) or source_node == target_node:
-                edit_script.extend(
-                    self._generate_move_edits(source_node, target_node, matching_set)
-                )
-                edit_script.append(Keep(source_node, target_node))
+
+            if (
+                not isinstance(source_node, UPDATABLE_EXPRESSION_TYPES)
+                or source_node == target_node
+            ):
+                edit_script.extend(self._generate_move_edits(source_node, target_node, matchings))
+
+                source_non_expression_leaves = dict(_get_non_expression_leaves(source_node))
+                target_non_expression_leaves = dict(_get_non_expression_leaves(target_node))
+
+                if source_non_expression_leaves != target_non_expression_leaves:
+                    edit_script.append(Update(source_node, target_node))
+                elif not delta_only:
+                    edit_script.append(Keep(source_node, target_node))
             else:
                 edit_script.append(Update(source_node, target_node))
 
         return edit_script
 
     def _generate_move_edits(
-        self, source: exp.Expression, target: exp.Expression, matching_set: t.Set[t.Tuple[int, int]]
+        self, source: exp.Expression, target: exp.Expression, matchings: t.Dict[int, int]
     ) -> t.List[Move]:
         source_args = [id(e) for e in _expression_only_args(source)]
         target_args = [id(e) for e in _expression_only_args(target)]
 
-        args_lcs = set(_lcs(source_args, target_args, lambda l, r: (l, r) in matching_set))
+        args_lcs = set(
+            _lcs(source_args, target_args, lambda l, r: matchings.get(t.cast(int, l)) == r)
+        )
 
         move_edits = []
         for a in source_args:
             if a not in args_lcs and a not in self._unmatched_source_nodes:
-                move_edits.append(Move(self._source_index[a]))
+                move_edits.append(
+                    Move(source=self._source_index[a], target=self._target_index[matchings[a]])
+                )
 
         return move_edits
 
@@ -199,10 +241,10 @@ class ChangeDistiller:
         matching_set = leaves_matching_set.copy()
 
         ordered_unmatched_source_nodes = {
-            id(n): None for n, *_ in self._source.bfs() if id(n) in self._unmatched_source_nodes
+            id(n): None for n in self._source.bfs() if id(n) in self._unmatched_source_nodes
         }
         ordered_unmatched_target_nodes = {
-            id(n): None for n, *_ in self._target.bfs() if id(n) in self._unmatched_target_nodes
+            id(n): None for n in self._target.bfs() if id(n) in self._unmatched_target_nodes
         }
 
         for source_node_id in ordered_unmatched_source_nodes:
@@ -210,8 +252,8 @@ class ChangeDistiller:
                 source_node = self._source_index[source_node_id]
                 target_node = self._target_index[target_node_id]
                 if _is_same_type(source_node, target_node):
-                    source_leaf_ids = {id(l) for l in _get_leaves(source_node)}
-                    target_leaf_ids = {id(l) for l in _get_leaves(target_node)}
+                    source_leaf_ids = {id(l) for l in _get_expression_leaves(source_node)}
+                    target_leaf_ids = {id(l) for l in _get_expression_leaves(target_node)}
 
                     max_leaves_num = max(len(source_leaf_ids), len(target_leaf_ids))
                     if max_leaves_num:
@@ -241,10 +283,10 @@ class ChangeDistiller:
 
     def _compute_leaf_matching_set(self) -> t.Set[t.Tuple[int, int]]:
         candidate_matchings: t.List[t.Tuple[float, int, int, exp.Expression, exp.Expression]] = []
-        source_leaves = list(_get_leaves(self._source))
-        target_leaves = list(_get_leaves(self._target))
-        for source_leaf in source_leaves:
-            for target_leaf in target_leaves:
+        source_expression_leaves = list(_get_expression_leaves(self._source))
+        target_expression_leaves = list(_get_expression_leaves(self._target))
+        for source_leaf in source_expression_leaves:
+            for target_leaf in target_expression_leaves:
                 if _is_same_type(source_leaf, target_leaf):
                     similarity_score = self._dice_coefficient(source_leaf, target_leaf)
                     if similarity_score >= self.f:
@@ -302,21 +344,30 @@ class ChangeDistiller:
         return bigram_histo
 
 
-def _get_leaves(expression: exp.Expression) -> t.Iterator[exp.Expression]:
+def _get_expression_leaves(expression: exp.Expression) -> t.Iterator[exp.Expression]:
     has_child_exprs = False
 
-    for _, node in expression.iter_expressions():
-        has_child_exprs = True
-        yield from _get_leaves(node)
+    for node in expression.iter_expressions():
+        if not isinstance(node, IGNORED_LEAF_EXPRESSION_TYPES):
+            has_child_exprs = True
+            yield from _get_expression_leaves(node)
 
     if not has_child_exprs:
         yield expression
 
 
+def _get_non_expression_leaves(expression: exp.Expression) -> t.Iterator[t.Tuple[str, t.Any]]:
+    for arg, value in expression.args.items():
+        if isinstance(value, exp.Expression) or (
+            isinstance(value, list) and isinstance(seq_get(value, 0), exp.Expression)
+        ):
+            continue
+
+        yield (arg, value)
+
+
 def _is_same_type(source: exp.Expression, target: exp.Expression) -> bool:
-    if type(source) is type(target) and (
-        not isinstance(source, exp.Identifier) or type(source.parent) is type(target.parent)
-    ):
+    if type(source) is type(target):
         if isinstance(source, exp.Join):
             return source.args.get("side") == target.args.get("side")
 
@@ -337,12 +388,12 @@ def _parent_similarity_score(
     return 1 + _parent_similarity_score(source.parent, target.parent)
 
 
-def _expression_only_args(expression: exp.Expression) -> t.List[exp.Expression]:
-    args: t.List[t.Union[exp.Expression, t.List]] = []
-    if expression:
-        for a in expression.args.values():
-            args.extend(ensure_list(a))
-    return [a for a in args if isinstance(a, exp.Expression)]
+def _expression_only_args(expression: exp.Expression) -> t.Iterator[exp.Expression]:
+    yield from (
+        arg
+        for arg in expression.iter_expressions()
+        if not isinstance(arg, IGNORED_LEAF_EXPRESSION_TYPES)
+    )
 
 
 def _lcs(
